#!/usr/bin/env -S perl -I./local/lib/perl5

use DBI;
use List::MoreUtils qw(uniq);
use Array::Utils qw(:all);
use DBD::SQLite::Constants qw/:file_open/;
use Data::Dumper;
use Encode;
use warnings;
use strict;

$|=1;

my $dbh = DBI->connect(
	"dbi:SQLite:dbname=$ENV{HOME}/Games/aoia/drive_c/users/mark/Local Settings/Application Data/Hallucina Software/AOIAPlus/aoitems.db",
	'',
	'',
	{sqlite_open_flags => SQLITE_OPEN_READONLY}
);

sub isFroobFriendly {
	my $nanoID = shift;

	my $sth = $dbh->prepare("
	SELECT
		COUNT(*) AS paidOnly
	FROM
		tblAOItemRequirements
	WHERE
			aoid=?
		AND
			(	(attribute=389 AND value>1 AND operator=22)
			OR	(attribute=54 AND value>=200 AND operator=2)
			OR	(attribute=169)
			OR	(attribute=182)
			OR	(aoid IN (301289))
			OR	(attribute=0 AND (value >= 3200 AND value <= 3217))
			OR	(attribute=355 AND value=64)
		)
	");
	$sth->execute($nanoID);
	my $row = $sth->fetchrow_hashref();
	return $row->{'paidOnly'} == 0;
}

my $sth = $dbh->prepare("
	SELECT
		n.aoid, n. name, n.type, n.description, n.flags, n.properties, n.icon, n.ncu,
		n.nanocost, n.duration, n.attack, n.recharge, n.range, n.initskill, n.strain AS strain_id,
		n.school AS school_id,
		c.crystal_id, c.crystal_name, c.crystal_icon, c.ql, c.type, c.school, c.strain, c.substrain,
		c.substrain_id, c.fp, c.sl, c.sl_spec, c.nanodeck, COALESCE(c.profession, 'NPC') AS profession,
		c.location,(
		   SELECT e.aoid
		     FROM tblAOItemEffects e
		     LEFT JOIN tblAOItems i ON (e.aoid=i.aoid)
		    WHERE     ((e.type IN (59, 74) AND e.value1 = n.aoid) OR (e.type IN (235) AND e.value2 = n.aoid))
		          AND e.aoid != 293113
		          AND NOT EXISTS(
		                SELECT * FROM tblAOItemRequirements r
		                 WHERE r.aoid=e.aoid
		                   AND attribute=455
		                   AND operator=2
		                   AND value >= 0
		          )
		ORDER BY COALESCE(i.flags,0) & 67108864 ASC,
		         e.aoid ASC
		   LIMIT 1
		) AS use_item,
		(SELECT SUM(r.value) FROM tblAOItemRequirements r WHERE r.aoid=n.aoid AND r.attribute IN (122,127,128,129,130,131)) AS sort_order
	FROM
		tblAONanos n
	LEFT JOIN
		tblAONanoCrystals c ON (c.nano_id=n.aoid)
");
$sth->execute();
my $nanos = {};
while (my $row = $sth->fetchrow_hashref()) {
	$row->{'froob_friendly'} = isFroobFriendly($row->{'crystal_id'});
	$row->{'profession'} = 'Use' if $row->{'use_item'} &&  !$row->{'crystal_id'} && index($row->{'name'} ,'_') == -1;
	if (exists($nanos->{$row->{'aoid'}})) {
		push(@{$nanos->{$row->{'aoid'}}->{'profession'}}, $row->{'profession'});
		$nanos->{$row->{'aoid'}}->{'profession'} = [uniq @{$nanos->{$row->{'aoid'}}->{'profession'}}];
		if ($nanos->{$row->{'aoid'}}->{'crystal_name'} !~ /^Nano/) {
			$nanos->{$row->{'aoid'}}->{'crystal_name'} = $row->{'crystal_name'};
			$nanos->{$row->{'aoid'}}->{'crystal_icon'} = $row->{'crystal_icon'};
			$nanos->{$row->{'aoid'}}->{'crystal_id'} = $row->{'crystal_id'};
		}
		$nanos->{$row->{'aoid'}}->{'froob_friedly'} ||= $row->{'froob_friendly'};
	} else {
		$row->{'profession'} = [$row->{'profession'}];
		$nanos->{$row->{'aoid'}} = $row;
	}
}

my $translateStrain = {
	"Fast 1s~3s" => "General Nuke",
	"Long 5s~4s" => "General Nuke",
	"Long 8s~1s" => "General Nuke",
	"Medium 3s~2s" => "General Nuke",
	"Medium 4s~3s" => "General Nuke",
	"Lost Eden" => "Orphan",
	"Shadowlands Disease Direct Damage Procs" => "Agent Damage Proc",
	"Aura Absorb/Reflect/AMS Buffs" => "Keeper Aura-Absorb/Reflect/AMS Buff",
	"Aura Damage/Snare Reduction Buffs" => "Keeper Aura-Damage/Snare Reduction Buff",
	"Aura HP and NP Regeneration Buffs" => "Keeper Aura-HP and NP Heal",
	"Proc Buffs" => "Keeper Proc Buff",
	"Strength/Stamina/Agility Buffs" => "Keeper Str/Stam/Agi Buff",
	"Biological Metamorphosis Buffs" => "BioMet Buff",
	"Matter Creation Buffs" => "MatCrea Buff",
	"Time and Space Buffs" => "MatLoc Buff",
	"Material Metamorphosis Buffs" => "MatMet Buff",
	"Nano Point Cost Buffs" => "NP Cost Buff",
	"Psychic and Intelligence/Nano Skills Buffs" => "Psy/Int Buffs",
	"Psychological Modification Buffs" => "PsyMod Buff",
	"Sensory Improvement Buffs" => "SenseImp Buff",
	"Biological Metamorphosis DeBuffs" => "BioMet Debuff",
	"Matter Creation DeBuffs" => "MatCrea Debuff",
	"Time and Space DeBuffs" => "MatLoc Debuff",
	"Material Metamorphosis DeBuffs" => "MatMet Debuff",
	"Psychological Modification DeBuffs" => "PsyMod Debuff",
	"Sensory Improvement DeBuffs" => "SenseImp Debuff",
	"Meta-Physicist Damage DeBuffs - Line A" => "MP Damage Debuff Line A",
	"Meta-Physicist Damage DeBuffs - Line B" => "MP Damage Debuff Line B",
	"Meta-Physicist Initiatives/Damage DeBuffs" => "MetaPhysicist Damage Debuff",
	"Meta-Physicist Attack Pet Damage Type Changer" => "MP Attack Pet Damage Type",
	"Meta-Physicist Pet Damage Buffs" => "MP Pet Damage Buffs",
	"Meta-Physicist Pet Initiative Buffs" => "MP Pet Initiative Buffs",
	"Nano Formula Range Buffs" => "NF Range Buff",
	"Nano-Technician DOT - Strain A" => "DOT Nanotechnician Strain A",
	"Nano-Technician DOT - Strain B" => "DOT Nanotechnician Strain B",
	"Nano Resistance DeBuffs - Line A" => "Nano Resistance Debuff (Line A)",
	"Shade Proc Buffs" => "Shade Proc Buff",
	"Weapon Effect Proc" => "Weapon Effect Add-On 2",
	"Burst Buffs" => "Burst Buff",
	"Armor Buffs" => "Armor Buff",
	"Soldier Damage Base" => "Damage Buffs - Line A",
	"Shadowland Reflect Base" => "Reflect Shield",
	"Shadowlands Nano Point Drains" => "SL Nanopoint Drain",
	"AAD Drains" => "Trader AAD Drain",
	"AAO Drains" => "Trader AAO Drain",
	"Nano Point Buffs" => "NP Buff",
	"AC Transfer Target Buffs" => "Trader AC Transfer Target Buff (Redeem)",
};

my $translateNanos = {
	267916 => ["Orphans", "Orphan - NO STACKING"],
	302247 => ["Orphans", "Orphan - NO STACKING"],
	266305 => ["Orphans", "Orphan - NO STACKING"],
	267917 => ["Orphans", "Orphan - NO STACKING"],
	156255 => ["Orphans", "Orphan - NO STACKING"],
	26237 => [undef, "Runspeed Buffs", "Run Speed Buffs"],
	26705 => [undef, "Runspeed Buffs", "Run Speed Buffs"],
	26698 => [undef, "Runspeed Buffs", "Team Run Speed Buffs"],
	32126 => [undef, "Runspeed Buffs", "Team Run Speed Buffs"],
	266307 => ["Orphans", "NO STACKING", ""],
	116855 => ["Orphans", "NO STACKING", ""],
	116856 => ["Orphans", "NO STACKING", ""],
	302254 => ["Pet Buffs", "Siphon Box"],
	301888 => ["Pet Buffs", "Siphon Box"],
	266309 => ["PvP", "Orphan"],
	274369 => ["PvP", "Orphan"],
	93132 => [undef, undef, "Major Evasion Buffs"],
	93126 => [undef, undef, "Major Evasion Buffs"],
	93127 => [undef, undef, "Major Evasion Buffs"],
	93128 => [undef, undef, "Major Evasion Buffs"],
	93129 => [undef, undef, "Major Evasion Buffs"],
	93130 => [undef, undef, "Major Evasion Buffs"],
	93131 => [undef, undef, "Major Evasion Buffs"],
	93125 => [undef, undef, "Major Evasion Buffs"],
	266308 => ["PvP", "Orphan"],
	297632 => ["Creation", "Summon Item", ""],
	210319 => ["Buffs", "Keeper Deflect/Riposte Buff", ""],
	210317 => ["Buffs", "Keeper Deflect/Riposte Buff", ""],
	210315 => ["Buffs", "Keeper Deflect/Riposte Buff", ""],
	210313 => ["Buffs", "Keeper Deflect/Riposte Buff", ""],
	210311 => ["Buffs", "Keeper Deflect/Riposte Buff", ""],
	211170 => ["Buffs", "Keeper Evade/Dodge/Duck Buff", ""],
	211168 => ["Buffs", "Keeper Evade/Dodge/Duck Buff", ""],
	211166 => ["Buffs", "Keeper Evade/Dodge/Duck Buff", ""],
	211164 => ["Buffs", "Keeper Evade/Dodge/Duck Buff", ""],
	266311 => ["Orphans", "Orphan", "PvP"],
	28868 => ["Orphans", "Orphan"],
	28890 => ["Orphans", "Orphan"],
	28894 => ["Orphans", "Orphan"],
	120499 => ["Orphans", "Orphan"],
	273379 => ["Buffs", "Psy/Int Buff", "Self"],
	29309 => ["Buffs", "Psy/Int Buff", "Self"],
	220349 => ["Buffs", "Psy/Int Buff", "Self"],
	220347 => ["Buffs", "Psy/Int Buff", "Self"],
	220345 => ["Buffs", "Psy/Int Buff", "Target"],
	266300 => ["Orphans", "Nemesis Nano Programs", ""],
	297342 => ["Orphans", "Spirit Drain", ""],
	301160 => ["Orphans", "Emergency Sneak", ""],
	301593 => ["Orphans", "AAD Buffs", ""],
	223199 => [undef, undef, "Soldier Damage Base"],
	223197 => [undef, undef, "Soldier Damage Base"],
	223195 => [undef, undef, "Soldier Damage Base"],
	223193 => [undef, undef, "Soldier Damage Base"],
	223191 => [undef, undef, "Soldier Damage Base"],
	223189 => [undef, undef, "Soldier Damage Base"],
	223187 => [undef, undef, "Soldier Damage Base"],
	222838 => [undef, undef, "Target"],
	222837 => [undef, undef, "Target"],
	222835 => [undef, undef, "Target"],
	222833 => [undef, undef, "Target"],
	233033 => [undef, undef, "Shadowland Reflect Base"],
	233029 => [undef, undef, "Shadowland Reflect Base"],
	233025 => [undef, undef, "Shadowland Reflect Base"],
	233022 => [undef, undef, "Shadowland Reflect Base"],
	233020 => [undef, undef, "Shadowland Reflect Base"],
	233018 => [undef, undef, "Shadowland Reflect Base"],
	233051 => [undef, undef, "Shadowland Reflect Base"],
	233049 => [undef, undef, "Shadowland Reflect Base"],
	233047 => [undef, undef, "Shadowland Reflect Base"],
	233045 => [undef, undef, "Shadowland Reflect Base"],
	233043 => [undef, undef, "Shadowland Reflect Base"],
	233041 => [undef, undef, "Shadowland Reflect Base"],
	233039 => [undef, undef, "Shadowland Reflect Base"],
	233037 => [undef, undef, "Shadowland Reflect Base"],
	233016 => [undef, undef, "Shadowland Reflect Base"],
	233014 => [undef, undef, "Shadowland Reflect Base"],
	233012 => [undef, undef, "Shadowland Reflect Base"],
	233035 => [undef, undef, "Shadowland Reflect Base"],
	29245 => ["Orphans", "Orphan", ""],
	302399 => ["Orphans", "Siphon Box", ""],
	302395 => ["Orphans", "Siphon Box", ""],
	267624 => ["Orphans", "Orphan", ""],
	266310 => ["Orphans", "Orphan", ""],
	302401 => ["Drain", "Drain Heal", ""],
	226746 => ["DeBuffs", "Nano Resistance Debuff (Line A)", ""],
	226742 => ["DeBuffs", "Nano Resistance Debuff (Line A)", ""],
	226738 => ["DeBuffs", "Nano Resistance Debuff (Line A)", ""],
	226734 => ["DeBuffs", "Nano Resistance Debuff (Line A)", ""],
	226728 => ["DeBuffs", "Nano Resistance Debuff (Line A)", ""],
	226726 => ["DeBuffs", "Nano Resistance Debuff (Line A)", ""],
	226748 => ["DeBuffs", "Debuff NanoAC Heavy", ""],
	226744 => ["DeBuffs", "Debuff NanoAC Heavy", ""],
	226740 => ["DeBuffs", "Debuff NanoAC Heavy", ""],
	226736 => ["DeBuffs", "Debuff NanoAC Heavy", ""],
	226730 => ["DeBuffs", "Debuff NanoAC Heavy", ""],
	226750 => ["DeBuffs", "Debuff NanoAC Heavy", ""],
	302288 => ["Orphans", "Trader Shudown Skill Debuff", ""],
	302860 => ["Orphans", "Trader Shudown Skill Buff", ""],
	302290 => ["Orphans", "Trader Shudown Skill Buff", ""],
	91316 => ["Drain", "Trader AC Transfer Target Debuff (Draw)", ""],
	91319 => ["Drain", "Trader AC Transfer Target Debuff (Draw)", ""],
	91322 => ["Drain", "Trader AC Transfer Target Debuff (Draw)", ""],
	91323 => ["Drain", "Trader AC Transfer Target Debuff (Draw)", ""],
	91328 => ["Drain", "Trader AC Transfer Target Debuff (Draw)", ""],
	91331 => ["Drain", "Trader AC Transfer Target Debuff (Draw)", ""],
	91334 => ["Drain", "Trader AC Transfer Target Debuff (Draw)", ""],
	91314 => ["Drain", "Trader AC Transfer Target Debuff (Siphon)", ""],
	91317 => ["Drain", "Trader AC Transfer Target Debuff (Siphon)", ""],
	91320 => ["Drain", "Trader AC Transfer Target Debuff (Siphon)", ""],
	91325 => ["Drain", "Trader AC Transfer Target Debuff (Siphon)", ""],
	91326 => ["Drain", "Trader AC Transfer Target Debuff (Siphon)", ""],
	91329 => ["Drain", "Trader AC Transfer Target Debuff (Siphon)", ""],
	91332 => ["Drain", "Trader AC Transfer Target Debuff (Siphon)", ""],
	91335 => ["Drain", "Trader AC Transfer Target Debuff (Siphon)", ""],

	275030 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	267114 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	302864 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85846 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85848 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85850 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85852 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85854 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85856 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85858 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85860 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85862 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85864 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85866 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85868 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85870 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],
	85872 => ["Drain", "Trader Skill Transfer Target Debuff (Deprive)", ""],

	275029 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	267110 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85845 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85847 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85849 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85851 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85853 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85855 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85857 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85859 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85861 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85863 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85865 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85867 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85869 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],
	85871 => ["Drain", "Trader Skill Transfer Target Debuff (Ransack)", ""],

	118245 => [undef, undef, "Team"],
	118230 => [undef, undef, "Team"],
	118232 => [undef, undef, "Team"],
	118231 => [undef, undef, "Team"],
	118235 => [undef, undef, "Team"],
	118233 => [undef, undef, "Team"],
	118234 => [undef, undef, "Team"],
	118238 => [undef, undef, "Team"],
	118236 => [undef, undef, "Team"],
	118237 => [undef, undef, "Team"],
	118241 => [undef, undef, "Team"],
	118239 => [undef, undef, "Team"],
	118240 => [undef, undef, "Team"],
	118243 => [undef, undef, "Team"],
	118244 => [undef, undef, "Team"],
	118242 => [undef, undef, "Team"],
	43374 => [undef, undef, "Team"],

	270357 => ["Drain", "Health Drain", ""],
	77195 => ["Drain", "Health Drain", ""],
	76478 => ["Drain", "Health Drain", ""],
	76475 => ["Drain", "Health Drain", ""],
	76487 => ["Drain", "Health Drain", ""],
	76481 => ["Drain", "Health Drain", ""],
	76484 => ["Drain", "Health Drain", ""],
	76491 => ["Drain", "Health Drain", ""],
	76494 => ["Drain", "Health Drain", ""],
	76499 => ["Drain", "Health Drain", ""],
	76571 => ["Drain", "Health Drain", ""],
	76503 => ["Drain", "Health Drain", ""],
	76651 => ["Drain", "Health Drain", ""],
	76614 => ["Drain", "Health Drain", ""],
	76656 => ["Drain", "Health Drain", ""],
	76653 => ["Drain", "Health Drain", ""],
	76679 => ["Drain", "Health Drain", ""],
	76681 => ["Drain", "Health Drain", ""],
	76684 => ["Drain", "Health Drain", ""],
	76686 => ["Drain", "Health Drain", ""],
	76691 => ["Drain", "Health Drain", ""],
	76688 => ["Drain", "Health Drain", ""],
	76717 => ["Drain", "Health Drain", ""],
	76715 => ["Drain", "Health Drain", ""],
	76720 => ["Drain", "Health Drain", ""],
	76722 => ["Drain", "Health Drain", ""],
	76724 => ["Drain", "Health Drain", ""],
	76727 => ["Drain", "Health Drain", ""],
	76729 => ["Drain", "Health Drain", ""],
	76732 => ["Drain", "Health Drain", ""],
	76742 => ["Drain", "Health Drain", ""],
	81879 => [undef, undef, "Target"],
	81880 => [undef, undef, "Target"],
	81881 => [undef, undef, "Target"],
	81882 => [undef, undef, "Target"],
	81883 => [undef, undef, "Target"],
	81884 => [undef, undef, "Target"],
	31448 => [undef, undef, "Target"],
	81885 => [undef, undef, "Target"],
	31411 => [undef, undef, "Target"],
	81886 => [undef, undef, "Target"],
	31384 => [undef, undef, "Target"],
	81887 => [undef, undef, "Target"],
	31401 => [undef, undef, "Target"],

	43367 => [undef, undef, "Target"],
	28892 => [undef, undef, "Target"],
	27188 => [undef, undef, "Target"],
	29764 => [undef, undef, "Target"],
	27189 => [undef, undef, "Target"],
	81829 => [undef, undef, "Target"],
	29758 => [undef, undef, "Target"],
	81830 => [undef, undef, "Target"],
	81853 => [undef, undef, "Target"],
	81826 => [undef, undef, "Target"],
	28876 => [undef, undef, "Target"],
	81852 => [undef, undef, "Target"],
	81823 => [undef, undef, "Target"],
	81922 => [undef, undef, "Target"],
	81824 => [undef, undef, "Target"],
	81851 => [undef, undef, "Target"],
	81822 => [undef, undef, "Target"],
	81856 => [undef, undef, "Target"],
	81825 => [undef, undef, "Target"],
	29760 => [undef, undef, "Target"],
	81827 => [undef, undef, "Target"],
	81855 => [undef, undef, "Target"],
	81854 => [undef, undef, "Target"],

	269463 => [undef, undef, "Self"],
	273288 => [undef, undef, "Self"],
	270798 => [undef, undef, "Self"],
};

print("DROP TABLE IF EXISTS nanos;\n");
print("CREATE TABLE nanos (\n".
	"\tcrystal_id INT DEFAULT NULL,\n".
	"\tuse_id INT DEFAULT NULL,\n".
	"\taoid INT DEFAULT NULL,\n".
	"\tql INT DEFAULT NULL,\n".
	"\tcrystal_name VARCHAR(255) DEFAULT NULL,\n".
	"\tnano_name VARCHAR(255) DEFAULT NULL,\n".
	"\tschool VARCHAR(255) DEFAULT NULL,\n".
	"\tschool_id INT DEFAULT NULL,\n".
	"\tstrain VARCHAR(255) DEFAULT NULL,\n".
	"\tstrain_id INT DEFAULT NULL,\n".
	"\tsub_strain VARCHAR(255) DEFAULT NULL,\n".
	"\tprofessions VARCHAR(255) DEFAULT NULL,\n".
	"\tlocation VARCHAR(255) DEFAULT NULL,\n".
	"\tnano_cost INT DEFAULT NULL,\n".
	"\tfroob_friendly BOOLEAN DEFAULT NULL,\n".
	"\tsort_order INT DEFAULT NULL\n".
");\n"
);
$nanos->{268610}->{'location'} = "Veteran Terminal (Tower Shop)";
my $sortedNanos = [sort { $a->{'name'} cmp $b->{'name'} } values %$nanos];
my @slProfs = ("Keeper", "Shade");
print("INSERT INTO nanos(crystal_id, use_id, aoid, ql, crystal_name, nano_name, school, school_id, strain, strain_id, sub_strain, professions, location, nano_cost, froob_friendly, sort_order) VALUES\n");
foreach my $nano (@$sortedNanos) {
	if (defined($nano->{'school'})) {
		$nano->{'school'} =~ s/_/ /g;
		$nano->{'school'} =~ s/DeBuff/Debuff/g;
		$nano->{'school'} = 'Orphans' if $nano->{'school'} =~ /PvP.*Nemesis/;
	} else {
		$nano->{'strain'} = '';
	}
	if (defined($nano->{'strain'})) {
		$nano->{'strain'} =~ s/_/ /g;
		$nano->{'strain'} =~ s/Add All Offen[cs]e/AAO/g;
		$nano->{'strain'} =~ s/Add All Defen[cs]e/AAD/g;
		$nano->{'strain'} =~ s/DeBuff/Debuff/g;
	} else {
		$nano->{'strain'} = '';
	}
	$nano->{'substrain'} =~ s/_/ /g if defined($nano->{'substrain'});
	$nano->{'substrain'} =~ s/DeBuff/Debuff/g if defined($nano->{'substrain'});
	$nano->{'strain'} = 'Orphan' if $nano->{'strain'} eq '' || !defined($nano->{'strain'});
	$nano->{'strain'} = $translateStrain->{$nano->{'strain'}} if exists($translateStrain->{$nano->{'strain'}});
	if (exists($translateNanos->{$nano->{'aoid'}})) {
		my $trans = $translateNanos->{$nano->{'aoid'}};
		$nano->{'school'} = $trans->[0] if defined($trans->[0]);
		$nano->{'strain'} = $trans->[1] if defined($trans->[1]);
		$nano->{'substrain'} = $trans->[2] if defined($trans->[2]);
	}
	my $quotedNanoName = $dbh->quote($nano->{'name'});
	$quotedNanoName =~ s/\\"/"/g;
	my $quotedCrystalName = $dbh->quote($nano->{'crystal_name'});
	$quotedCrystalName =~ s/\\"/"/g;
	printf(
		"(%s, %s, %s, %s, %s, %s, %s, %d, %s, %d, %s, %s, %s, %s, %s, %s),\n",
		$nano->{'crystal_id'} ? $nano->{'crystal_id'} : 'NULL',
		$nano->{'use_item'} ? $nano->{'use_item'} : 'NULL',
		$nano->{'aoid'} ? $nano->{'aoid'} : 'NULL',
		$nano->{'ql'} ? $nano->{'ql'} : 'NULL',
		$quotedCrystalName,
		$quotedNanoName,
		$dbh->quote($nano->{'school'}),
		$nano->{'school_id'},
		$dbh->quote($nano->{'strain'}),
		$nano->{'strain_id'},
		$dbh->quote($nano->{'substrain'}),
		$dbh->quote(join(':', @{$nano->{'profession'}})),
		$dbh->quote($nano->{'location'}),
		$nano->{'nanocost'} ? $nano->{'nanocost'} : 'NULL',
		(!$nano->{'sl'} && !$nano->{'sl_spec'} && !$nano->{'nanodeck'} && $nano->{'froob_friendly'} && scalar(array_minus(@{$nano->{'profession'}}, @slProfs))) ? 'true' : 'false',
		$nano->{'sort_order'} ? $nano->{'sort_order'} : 'NULL',
	);
	#print(Dumper($nano));
}
